import 'package:dartex/dartex.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/rendering.dart';
import 'package:ordered_set/comparing.dart';
import 'package:ordered_set/ordered_set.dart';

abstract class RenderCommand {
  final Matrix4 transform;

  RenderCommand({@required this.transform});

  void render(Canvas canvas);
}

abstract class CameraCommand extends RenderCommand {
  CameraCommand({@required Matrix4 transform}) : super(transform: transform);
}

class Renderer with Resource<Renderer> {
  final OrderedSet<RenderCommand> _renderCommands;

  CameraCommand _cameraCommand;

  Renderer()
      : _renderCommands = OrderedSet(
          Comparing.on((command) => command.transform.getTranslation().z),
        );

  void submit(RenderCommand command) {
    if (command is CameraCommand) {
      _cameraCommand = command;
    } else {
      _renderCommands.add(command);
    }
  }

  void render(Canvas canvas) {
    if (_cameraCommand != null) {
      _cameraCommand.render(canvas);
    }

    _renderCommands.forEach((command) {
      canvas.save();
      canvas.transform(command.transform.storage);

      command.render(canvas);

      canvas.restore();
    });

    _renderCommands.clear();
  }
}
