import 'package:dartex/dartex.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/rendering.dart' hide Size;
import 'package:fluttershy/size.dart';
import 'package:fluttershy_backend_dartex/components/camera.dart';
import 'package:fluttershy_backend_dartex/components/local_to_world.dart';
import 'package:fluttershy_backend_dartex/resources/renderer.dart';
import 'package:fluttershy_backend_dartex/resources/screen_dimensions.dart';

class CameraRenderCommand extends CameraCommand {
  final Camera camera;
  final Size screenSize;

  CameraRenderCommand({
    @required this.camera,
    @required this.screenSize,
    @required Matrix4 transform,
  }) : super(transform: transform);

  @override
  void render(Canvas canvas) {
    if (camera.size.width != double.infinity &&
        camera.size.height != double.infinity) {
      canvas.scale(
        screenSize.width / camera.size.width,
        screenSize.height / camera.size.height,
      );
    } else if (camera.size.width == double.infinity &&
        camera.size.height != double.infinity) {
      canvas.scale(screenSize.height / camera.size.height);
    } else if (camera.size.width != double.infinity &&
        camera.size.height == double.infinity) {
      canvas.scale(screenSize.width / camera.size.width);
    }

    canvas.transform(Matrix4.inverted(transform).storage);
  }
}

class CameraSystem extends System {
  CameraSystem() : super(type: [Camera, LocalToWorld]);

  @override
  void run(World world, List<Entity> entities) {
    final renderer = world.getResource<Renderer>();
    final screenDimensions = world.getResource<ScreenDimensions>();

    entities.forEach((entity) {
      renderer.submit(CameraRenderCommand(
        camera: entity.getComponent<Camera>(),
        screenSize: screenDimensions.size,
        transform: entity.getComponent<LocalToWorld>().matrix,
      ));
    });
  }
}
