import 'dart:ui';

import 'package:dartex/dartex.dart';
import 'package:fluttershy_backend_dartex/components/rectangle.dart';
import 'package:fluttershy_backend_dartex/components/local_to_world.dart';
import 'package:fluttershy_backend_dartex/resources/renderer.dart';
import 'package:vector_math/vector_math_64.dart';

class RectangleRenderCommand extends RenderCommand {
  final Rectangle rectangle;

  RectangleRenderCommand({this.rectangle, Matrix4 transform})
      : super(transform: transform);

  @override
  void render(Canvas canvas) {
    canvas.drawRect(
      Rect.fromLTWH(0, 0, rectangle.size.width, rectangle.size.height),
      Paint()..color = rectangle.color,
    );
  }
}

class RectangleRenderingSystem extends System {
  RectangleRenderingSystem() : super(type: [Rectangle, LocalToWorld]);

  @override
  void run(World world, List<Entity> entities) {
    final renderer = world.getResource<Renderer>();

    entities.forEach((entity) {
      renderer.submit(RectangleRenderCommand(
        rectangle: entity.getComponent<Rectangle>(),
        transform: entity.getComponent<LocalToWorld>().matrix,
      ));
    });
  }
}
